//
//  ODOvalView.m
//  OvalDoc
//
//  Created by narita on 2013/10/05.
//  Copyright (c) 2013年 narita. All rights reserved.
//

#import "ODOvalView.h"

@implementation ODOvalView

+ (void) initialize
{
    // サブクラスからの呼び出しに対するガード
    if (self == [ODOvalView class])
    {
        [self exposeBinding:@"oval"];
    }
}

- (id)initWithFrame:(NSRect)frame
{
    self = [super initWithFrame:frame];
    
    if (self)
    {
        // Initialization code here.
    }
    
    return self;
}

#pragma mark Draw

- (void)drawRect:(NSRect)dirtyRect
{
    // Drawing code here.

    NSRect theBounds = [self bounds];
    
    [[NSColor greenColor] set];
    
    [NSBezierPath fillRect:theBounds];
    
    [[NSColor whiteColor] set];
    
    NSBezierPath* theBezier = [NSBezierPath bezierPathWithOvalInRect:_oval];
    
    [theBezier fill];
}

#pragma mark Events

- (void) mouseDown:(NSEvent *)theEvent
{
    NSPoint thePoint = [theEvent locationInWindow];
    
    self.downPoint = [self convertPoint:thePoint fromView:nil];
    
    self.currentPoint = self.downPoint;
    
    [self setNeedsDisplay:YES];
}

- (void) mouseDragged:(NSEvent *)theEvent
{
    NSPoint thePoint = [theEvent locationInWindow];
    
    self.currentPoint = [self convertPoint:thePoint fromView:nil];
    
    [self autoscroll:theEvent];
    
    [self setNeedsDisplay:YES];
}

- (void) mouseUp:(NSEvent *)theEvent
{
    NSPoint thePoint = [theEvent locationInWindow];
    
    self.currentPoint = [self convertPoint:thePoint fromView:nil];
    
    self.oval = [self currentRect];
}

- (NSRect) currentRect
{
    float theMinX = MIN(self.downPoint.x, self.currentPoint.x);
    float theMaxX = MAX(self.downPoint.x, self.currentPoint.x);
    float theMinY = MIN(self.downPoint.y, self.currentPoint.y);
    float theMaxY = MAX(self.downPoint.y, self.currentPoint.y);
    
    return NSMakeRect(theMinX,
                      theMinY,
                      theMaxX - theMinX,
                      theMaxY - theMinY);
}

#pragma mark Accessors

@synthesize oval = _oval;

- (NSRect) oval
{
    return _oval;
}

- (void) setOval:(NSRect)inOval
{
    if( !NSEqualRects(_oval, inOval) )
    {
        _oval = inOval;
        
        // binding元への伝播
        NSDictionary* theInfo = [self infoForBinding:@"oval"];
        
        if( theInfo )
        {
            NSString*     thePath    = [theInfo valueForKey:NSObservedKeyPathKey];
            NSString*     theObject  = [theInfo valueForKey:NSObservedObjectKey];
//            NSDictionary* theOptions = [theInfo valueForKey:NSOptionsKey];
            
            [theObject setValue:[NSValue valueWithRect:_oval]
                     forKeyPath:thePath];
        }
        
        
        [self setNeedsDisplay:YES];
    }
}

@end
